@@ -3,6 +3,7 @@ package App::Netdisco::Web::Plugin::Report::NodeVendor;
 use Dancer ':syntax';
 use Dancer::Plugin::DBIC;
 use Dancer::Plugin::Auth::Extensible;
+use App::Netdisco::Util::ExpandParams 'expand_hash';
 
 use App::Netdisco::Web::Plugin;
 
@@ -14,20 +15,6 @@ register_report(
     }
 );
 
-hook 'before' => sub {
-
-    return
-        unless (
-        request->path eq uri_for('/report/nodevendor')->path
-        or index( request->path,
-            uri_for('/ajax/content/report/nodevendor')->path ) == 0
-        );
-
-    params->{'limit'} ||= 1024;
-    params->{'order'} ||= 'MAC';
-
-};
-
 hook 'before_template' => sub {
     my $tokens = shift;
 
@@ -49,54 +36,96 @@ hook 'before_template' => sub {
     }
 };
 
-get '/ajax/content/report/nodevendor' => require_login sub {
+get '/ajax/content/report/nodevendor/data' => require_login sub {
+    send_error( 'Missing parameter', 400 )
+        unless ( param('draw') && param('draw') =~ /\d+/ );
 
     my $vendor = param('vendor');
 
     my $rs = schema('netdisco')->resultset('Node');
 
-    if ( defined $vendor ) {
-
         my $match = $vendor eq 'blank' ? undef : $vendor;
 
-        my $order = {
-            MAC    => 'me.mac',
-            Device => 'me.switch',
-            Vendor => 'oui.company'
-        };
-
-        $rs = $rs->search( { 'oui.abbrev' => $match } )
-            ->prefetch( [qw/ oui device /] );
+        $rs = $rs->search( { 'oui.abbrev' => $match },
+            {   '+columns' => [qw/ device.dns device.name oui.abbrev /],
+                join       => [qw/ oui device /],
+                collapse   => 1,
+            });
 
         unless ( param('archived') ) {
             $rs = $rs->search( { -bool => 'me.active' } );
         }
 
-        $rs = $rs->order_by( $order->{ param('order') } )
-            ->limit( param('limit') )->hri;
+    my $exp_params = expand_hash( scalar params );
+
+    my $recordsTotal = $rs->count;
+
+    my @data = $rs->get_datatables_data($exp_params)->hri->all;
+
+    my $recordsFiltered = $rs->get_datatables_filtered_count($exp_params);
+
+    content_type 'application/json';
+    return to_json(
+        {   draw            => int( param('draw') ),
+            recordsTotal    => int($recordsTotal),
+            recordsFiltered => int($recordsFiltered),
+            data            => \@data,
+        }
+    );
+};
+
+get '/ajax/content/report/nodevendor' => require_login sub {
+
+    my $vendor = param('vendor');
+
+    my $rs = schema('netdisco')->resultset('Node');
+    my @results;
+    
+    if ( defined $vendor && !request->is_ajax ) {
+
+        my $match = $vendor eq 'blank' ? undef : $vendor;
+
+        $rs = $rs->search( { 'oui.abbrev' => $match },
+            {   '+columns' => [qw/ device.dns device.name oui.abbrev /],
+                join       => [qw/ oui device /],
+                collapse   => 1,
+            });
+
+        unless ( param('archived') ) {
+            $rs = $rs->search( { -bool => 'me.active' } );
+        }
+
+        @results = $rs->hri->all;
+        return unless scalar @results;
     }
-    else {
+    elsif ( !defined $vendor ) {
         $rs = $rs->search(
-            { -bool => 'me.active' },
+            { },
             {   join     => 'oui',
                 select   => [ 'oui.abbrev', { count => 'me.mac' } ],
                 as       => [qw/ vendor count /],
                 group_by => [qw/ oui.abbrev /]
             }
-        )->order_by( { -desc => 'count' } )->hri;
+        )->order_by( { -desc => 'count' } );
+
+        unless ( param('archived') ) {
+            $rs = $rs->search( { -bool => 'me.active' } );
+        }
+        
+        @results = $rs->hri->all;
+        return unless scalar @results;
     }
 
-    return unless $rs->has_rows;
-
     if ( request->is_ajax ) {
+        my $json = to_json( \@results );
         template 'ajax/report/nodevendor.tt',
-            { results => $rs, opt => $vendor },
+            { results => $json, opt => $vendor },
             { layout => undef };
     }
     else {
         header( 'Content-Type' => 'text/comma-separated-values' );
         template 'ajax/report/nodevendor_csv.tt',
-            { results => $rs, opt => $vendor },
+            { results => \@results, opt => $vendor },
             { layout => undef };
     }
 };
